@@ -1,7 +1,7 @@
 
 ## fastLm.R: Rcpp/Armadillo implementation of lm()
 ##
-## Copyright (C)  2010 - 2011  Dirk Eddelbuettel, Romain Francois and Douglas Bates
+## Copyright (C)  2010 - 2012  Dirk Eddelbuettel, Romain Francois and Douglas Bates
 ##
 ## This file is part of RcppArmadillo.
 ##
@@ -20,8 +20,7 @@
 
 fastLmPure <- function(X, y) {
 
-    stopifnot(is.matrix(X))
-    stopifnot(nrow(y)==nrow(X))
+    stopifnot(is.matrix(X), is.numeric(y), nrow(y)==nrow(X))
 
     .Call("fastLm", X, y, package = "RcppArmadillo")
 }
@@ -42,6 +41,7 @@ fastLm.default <- function(X, y, ...) {
     res$fitted.values <- as.vector(X %*% res$coefficients)
     res$residuals <- y - res$fitted.values
     res$call <- match.call()
+    res$intercept <- any(apply(X, 2, function(x) all(x == x[1])))
 
     class(res) <- "fastLm"
     res
@@ -65,16 +65,18 @@ summary.fastLm <- function(object, ...) {
 
     # why do I need this here?
     rownames(TAB) <- names(object$coefficients)
-#    colnames(TAB) <- c("Estimate", "StdErr", "t.value", "p.value")
+    colnames(TAB) <- c("Estimate", "StdErr", "t.value", "p.value")
 
-    ## cf src/stats/R/lm.R and case with no weights and an intercept
+    ## cf src/library/stats/R/lm.R and case with no weights and an intercept
     f <- object$fitted.values
     r <- object$residuals
-    mss <- sum((f - mean(f))^2)
+    #mss <- sum((f - mean(f))^2)
+    mss <- if (object$intercept) sum((f - mean(f))^2) else sum(f^2)
     rss <- sum(r^2)
 
     r.squared <- mss/(mss + rss)
-    df.int <- 1 		# case of intercept
+    df.int <- if (object$intercept) 1L else 0L
+
     n <- length(f)
     rdf <- object$df
     adj.r.squared <- 1 - (1 - r.squared) * ((n - df.int)/rdf)
@@ -82,7 +84,10 @@ summary.fastLm <- function(object, ...) {
     res <- list(call=object$call,
                 coefficients=TAB,
                 r.squared=r.squared,
-                adj.r.squared=adj.r.squared)
+                adj.r.squared=adj.r.squared,
+                sigma=sqrt(sum((object$residuals)^2)/rdf),
+                df=object$df,
+                residSum=summary(object$residuals, digits=5)[-4])
 
     class(res) <- "summary.fastLm"
     res
@@ -91,10 +96,14 @@ summary.fastLm <- function(object, ...) {
 print.summary.fastLm <- function(x, ...) {
     cat("\nCall:\n")
     print(x$call)
+    cat("\nResiduals:\n")
+    print(x$residSum)
     cat("\n")
 
     printCoefmat(x$coefficients, P.values=TRUE, has.Pvalue=TRUE)
     digits <- max(3, getOption("digits") - 3)
+    cat("\nResidual standard error: ", formatC(x$sigma, digits=digits), " on ",
+        formatC(x$df), " degrees of freedom\n", sep="")
     cat("Multiple R-squared: ", formatC(x$r.squared, digits=digits),
         ",\tAdjusted R-squared: ",formatC(x$adj.r.squared, digits=digits),
         "\n", sep="")
@@ -109,6 +118,7 @@ fastLm.formula <- function(formula, data=list(), ...) {
     res <- fastLm.default(x, y, ...)
     res$call <- match.call()
     res$formula <- formula
+    res$intercept <- attr(attr(mf, "terms"), "intercept")
     res
 }
 
