@@ -100,6 +100,33 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
 
   inline bool is_vec() const;
 
+  inline       SpSubview row(const uword row_num);
+  inline const SpSubview row(const uword row_num) const;
+
+  inline       SpSubview col(const uword col_num);
+  inline const SpSubview col(const uword col_num) const;
+
+  inline       SpSubview rows(const uword in_row1, const uword in_row2);
+  inline const SpSubview rows(const uword in_row1, const uword in_row2) const;
+
+  inline       SpSubview cols(const uword in_col1, const uword in_col2);
+  inline const SpSubview cols(const uword in_col1, const uword in_col2) const;
+
+  inline       SpSubview submat(const uword in_row1, const uword in_col1, const uword in_row2, const uword in_col2);
+  inline const SpSubview submat(const uword in_row1, const uword in_col1, const uword in_row2, const uword in_col2) const;
+
+  inline       SpSubview submat(const span& row_span, const span& col_span);
+  inline const SpSubview submat(const span& row_span, const span& col_span) const;
+
+  inline       SpSubview operator()(const uword row_num, const span& col_span);
+  inline const SpSubview operator()(const uword row_num, const span& col_span) const;
+
+  inline       SpSubview operator()(const span& row_span, const uword col_num);
+  inline const SpSubview operator()(const span& row_span, const uword col_num) const;
+
+  inline       SpSubview operator()(const span& row_span, const span& col_span);
+  inline const SpSubview operator()(const span& row_span, const span& col_span) const;
+
 /* not yet
   inline       SpSubview_row<eT> row(const uword row_num);
   inline const SpSubview_row<eT> row(const uword row_num) const;
@@ -138,7 +165,14 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
   inline void swap_rows(const uword in_row1, const uword in_row2);
   inline void swap_cols(const uword in_col1, const uword in_col2);
 
-  // Similar to SpMat iterators but automatically iterate over values not in the subview.
+  // Forward declarations.
+  class iterator_base;
+  class const_iterator;
+  class iterator;
+  class const_row_iterator;
+  class row_iterator;
+
+  // Similar to SpMat iterators but automatically iterates past and ignores values not in the subview.
   class iterator_base
     {
     public:
@@ -148,12 +182,6 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
 
     inline eT operator*() const;
 
-    inline bool operator!=(const iterator_base& rhs) const;
-    inline bool operator==(const iterator_base& rhs) const;
-
-    inline bool operator!=(const typename SpMat<eT>::iterator_base& rhs) const;
-    inline bool operator==(const typename SpMat<eT>::iterator_base& rhs) const;
-
     // Don't hold location internally; call "dummy" methods to get that information.
     arma_inline uword row() const { return M.m.row_indices[internal_pos + skip_pos] - M.aux_row1; }
     arma_inline uword col() const { return internal_col;                                          }
@@ -163,6 +191,13 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
     arma_aligned       uword      internal_col;
     arma_aligned       uword      internal_pos;
     arma_aligned       uword      skip_pos; // not used in row_iterator or const_row_iterator
+
+    // So that we satisfy the STL iterator types.
+    typedef std::bidirectional_iterator_tag iterator_category;
+    typedef eT                              value_type;
+    typedef uword                           difference_type; // not certain on this one
+    typedef const eT*                       pointer;
+    typedef const eT&                       reference;
     };
 
   class const_iterator : public iterator_base
@@ -175,10 +210,22 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
     inline const_iterator(const const_iterator& other);
 
     inline const_iterator& operator++();
-    inline void            operator++(int);
+    inline const_iterator  operator++(int);
 
     inline const_iterator& operator--();
-    inline void            operator--(int);
+    inline const_iterator  operator--(int);
+
+    inline bool operator!=(const const_iterator& rhs) const;
+    inline bool operator==(const const_iterator& rhs) const;
+
+    inline bool operator!=(const typename SpMat<eT>::const_iterator& rhs) const;
+    inline bool operator==(const typename SpMat<eT>::const_iterator& rhs) const;
+
+    inline bool operator!=(const const_row_iterator& rhs) const;
+    inline bool operator==(const const_row_iterator& rhs) const;
+
+    inline bool operator!=(const typename SpMat<eT>::const_row_iterator& rhs) const;
+    inline bool operator==(const typename SpMat<eT>::const_row_iterator& rhs) const;
     };
 
   class iterator : public const_iterator
@@ -194,10 +241,15 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
 
     // overloads needed for return type correctness
     inline iterator& operator++();
-    inline void      operator++(int);
+    inline iterator  operator++(int);
 
     inline iterator& operator--();
-    inline void      operator--(int);
+    inline iterator  operator--(int);
+
+    // This has a different value_type than iterator_base.
+    typedef SpValProxy<SpSubview<eT> >        value_type;
+    typedef const SpValProxy<SpSubview<eT> >* pointer;
+    typedef const SpValProxy<SpSubview<eT> >& reference;
     };
 
   class const_row_iterator : public iterator_base
@@ -209,10 +261,10 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
     inline const_row_iterator(const const_row_iterator& other);
 
     inline const_row_iterator& operator++();
-    inline void                operator++(int);
+    inline const_row_iterator  operator++(int);
 
     inline const_row_iterator& operator--();
-    inline void                operator--(int);
+    inline const_row_iterator  operator--(int);
 
     uword internal_row; // Hold row internally because we use internal_pos differently.
     uword actual_pos; // Actual position in subview's parent matrix.
@@ -220,6 +272,18 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
     arma_inline eT operator*() const { return iterator_base::M.m.values[actual_pos]; }
 
     arma_inline uword row() const { return internal_row; }
+
+    inline bool operator!=(const const_iterator& rhs) const;
+    inline bool operator==(const const_iterator& rhs) const;
+
+    inline bool operator!=(const typename SpMat<eT>::const_iterator& rhs) const;
+    inline bool operator==(const typename SpMat<eT>::const_iterator& rhs) const;
+
+    inline bool operator!=(const const_row_iterator& rhs) const;
+    inline bool operator==(const const_row_iterator& rhs) const;
+
+    inline bool operator!=(const typename SpMat<eT>::const_row_iterator& rhs) const;
+    inline bool operator==(const typename SpMat<eT>::const_row_iterator& rhs) const;
     };
 
   class row_iterator : public const_row_iterator
@@ -234,10 +298,15 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
 
     // overloads needed for return type correctness
     inline row_iterator& operator++();
-    inline void          operator++(int);
+    inline row_iterator  operator++(int);
 
     inline row_iterator& operator--();
-    inline void          operator--(int);
+    inline row_iterator  operator--(int);
+
+    // This has a different value_type than iterator_base.
+    typedef SpValProxy<SpSubview<eT> >        value_type;
+    typedef const SpValProxy<SpSubview<eT> >* pointer;
+    typedef const SpValProxy<SpSubview<eT> >& reference;
     };
 
   inline iterator           begin();
@@ -246,8 +315,8 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
   inline iterator           begin_col(const uword col_num);
   inline const_iterator     begin_col(const uword col_num) const;
 
-  inline row_iterator       begin_row();
-  inline const_row_iterator begin_row() const;
+  inline row_iterator       begin_row(const uword row_num = 0);
+  inline const_row_iterator begin_row(const uword row_num = 0) const;
 
   inline iterator           end();
   inline const_iterator     end() const;
@@ -255,6 +324,9 @@ class SpSubview : public SpBase<eT, SpSubview<eT> >
   inline row_iterator       end_row();
   inline const_row_iterator end_row() const;
 
+  inline row_iterator       end_row(const uword row_num = 0);
+  inline const_row_iterator end_row(const uword row_num = 0) const;
+
 
   private:
   friend class SpMat<eT>;
