@@ -47,13 +47,11 @@
   {\
   if(n_rows != 1)\
     {\
-    uword count = 0;\
-    \
     for(uword col=0; col<n_cols; ++col)\
       {\
       uword i,j;\
       \
-      for(i=0, j=1; j<n_rows; i+=2, j+=2, count+=2)\
+      for(i=0, j=1; j<n_rows; i+=2, j+=2)\
         {\
         eT tmp_i = P.at(i,col);\
         eT tmp_j = P.at(j,col);\
@@ -61,14 +59,13 @@
         tmp_i = eop_core<eop_type>::process(tmp_i, k);\
         tmp_j = eop_core<eop_type>::process(tmp_j, k);\
         \
-        out_mem[count  ] operatorA tmp_i;\
-        out_mem[count+1] operatorA tmp_j;\
+        *out_mem operatorA tmp_i;  out_mem++;\
+        *out_mem operatorA tmp_j;  out_mem++;\
         }\
       \
       if(i < n_rows)\
         {\
-        out_mem[count] operatorA eop_core<eop_type>::process(P.at(i,col), k);\
-        ++count;\
+        *out_mem operatorA eop_core<eop_type>::process(P.at(i,col), k);  out_mem++;\
         }\
       }\
     }\
@@ -85,15 +82,13 @@
 
 #define arma_applier_3(operatorA) \
   {\
-  uword count = 0;\
-  \
   for(uword slice=0; slice<n_slices; ++slice)\
     {\
     for(uword col=0; col<n_cols; ++col)\
       {\
       uword i,j;\
       \
-      for(i=0, j=1; j<n_rows; i+=2, j+=2, count+=2)\
+      for(i=0, j=1; j<n_rows; i+=2, j+=2)\
         {\
         eT tmp_i = P.at(i,col,slice);\
         eT tmp_j = P.at(j,col,slice);\
@@ -101,14 +96,13 @@
         tmp_i = eop_core<eop_type>::process(tmp_i, k);\
         tmp_j = eop_core<eop_type>::process(tmp_j, k);\
         \
-        out_mem[count  ] operatorA tmp_i;\
-        out_mem[count+1] operatorA tmp_j;\
+        *out_mem operatorA tmp_i; out_mem++; \
+        *out_mem operatorA tmp_j; out_mem++; \
         }\
       \
       if(i < n_rows)\
         {\
-        out_mem[count] operatorA eop_core<eop_type>::process(P.at(i,col,slice), k);\
-        ++count;\
+        *out_mem operatorA eop_core<eop_type>::process(P.at(i,col,slice), k); out_mem++; \
         }\
       }\
     }\
