@@ -477,10 +477,10 @@ arma_mat_norm_inf(const Proxy<T1>& P)
 template<typename T1>
 inline
 arma_warn_unused
-typename T1::pod_type
+typename enable_if2< is_arma_type<T1>::value, typename T1::pod_type >::result
 norm
   (
-  const Base<typename T1::elem_type,T1>& X,
+  const T1& X,
   const uword k,
   const typename arma_float_or_cx_only<typename T1::elem_type>::result* junk = 0
   )
@@ -491,7 +491,7 @@ norm
   typedef typename T1::elem_type eT;
   typedef typename T1::pod_type   T;
   
-  const Proxy<T1> P(X.get_ref());
+  const Proxy<T1> P(X);
   
   if(P.get_n_elem() == 0)
     {
@@ -543,10 +543,10 @@ norm
 template<typename T1>
 inline
 arma_warn_unused
-typename T1::pod_type
+typename enable_if2< is_arma_type<T1>::value, typename T1::pod_type >::result
 norm
   (
-  const Base<typename T1::elem_type,T1>& X,
+  const T1& X,
   const char* method,
   const typename arma_float_or_cx_only<typename T1::elem_type>::result* junk = 0
   )
@@ -557,7 +557,7 @@ norm
   typedef typename T1::elem_type eT;
   typedef typename T1::pod_type   T;
   
-  const Proxy<T1> P(X.get_ref());
+  const Proxy<T1> P(X);
   
   if(P.get_n_elem() == 0)
     {
@@ -610,4 +610,212 @@ norm
 
 
 
+//
+// norms for sparse matrices
+
+
+
+template<typename T1>
+inline
+typename T1::pod_type
+arma_mat_norm_1(const SpProxy<T1>& P)
+  {
+  arma_extra_debug_sigprint();
+  
+  // TODO: this can be sped up with a dedicated implementation
+  return as_scalar( max( sum(abs(P.Q), 0), 1) );
+  }
+
+
+
+// template<typename T1>
+// inline
+// typename T1::pod_type
+// arma_mat_norm_2(const SpProxy<T1>& P)
+//   {
+//   arma_extra_debug_sigprint();
+//   
+//   // TODO: norm = sqrt( largest eigenvalue of (A^H)*A ), where ^H is the conjugate transpose
+//   // TODO: can use ARPACK or directly implement the Arnoldi iteration
+//   // http://math.stackexchange.com/questions/4368/computing-the-largest-eigenvalue-of-a-very-large-sparse-matrix
+//   }
+
+
+
+template<typename T1>
+inline
+typename T1::pod_type
+arma_mat_norm_inf(const SpProxy<T1>& P)
+  {
+  arma_extra_debug_sigprint();
+  
+  // TODO: this can be sped up with a dedicated implementation
+  return as_scalar( max( sum(abs(P.Q), 1), 0) );
+  }
+
+
+
+template<typename T1>
+inline
+arma_warn_unused
+typename enable_if2< is_arma_sparse_type<T1>::value, typename T1::pod_type >::result
+norm
+  (
+  const T1& X,
+  const uword k,
+  const typename arma_float_or_cx_only<typename T1::elem_type>::result* junk = 0
+  )
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  typedef typename T1::elem_type eT;
+  typedef typename T1::pod_type   T;
+  
+  const SpProxy<T1> P(X);
+  
+  if(P.get_n_nonzero() == 0)
+    {
+    return T(0);
+    }
+  
+  const bool is_vec = (P.get_n_rows() == 1) || (P.get_n_cols() == 1);
+  
+  if(is_vec == true)
+    {
+    const unwrap_spmat<typename SpProxy<T1>::stored_type> tmp(P.Q);
+    const SpMat<eT>& A = tmp.M;
+    
+    // create a fake dense vector to allow reuse of code for dense vectors
+    Col<eT> fake_vector( access::rwp(A.values), A.n_nonzero, false );
+    
+    const Proxy< Col<eT> > P_fake_vector(fake_vector);
+    
+    switch(k)
+      {
+      case 1:
+        return arma_vec_norm_1(P_fake_vector);
+        break;
+      
+      case 2:
+        return arma_vec_norm_2(P_fake_vector);
+        break;
+      
+      default:
+        {
+        arma_debug_check( (k == 0), "norm(): k must be greater than zero"   );
+        return arma_vec_norm_k(P_fake_vector, int(k));
+        }
+      }
+    }
+  else
+    {
+    switch(k)
+      {
+      case 1:
+        return arma_mat_norm_1(P);
+        break;
+      
+      // case 2:
+      //   return arma_mat_norm_2(P);
+      //   break;
+      
+      default:
+        arma_stop("norm(): unsupported or unimplemented norm type for sparse matrices");
+        return T(0);
+      }
+    }
+  }
+
+
+
+template<typename T1>
+inline
+arma_warn_unused
+typename enable_if2< is_arma_sparse_type<T1>::value, typename T1::pod_type >::result
+norm
+  (
+  const T1& X,
+  const char* method,
+  const typename arma_float_or_cx_only<typename T1::elem_type>::result* junk = 0
+  )
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk);
+  
+  typedef typename T1::elem_type eT;
+  typedef typename T1::pod_type   T;
+  
+  const SpProxy<T1> P(X);
+  
+  if(P.get_n_nonzero() == 0)
+    {
+    return T(0);
+    }
+  
+  
+  const unwrap_spmat<typename SpProxy<T1>::stored_type> tmp(P.Q);
+  const SpMat<eT>& A = tmp.M;
+  
+  // create a fake dense vector to allow reuse of code for dense vectors
+  Col<eT> fake_vector( access::rwp(A.values), A.n_nonzero, false );
+  
+  const Proxy< Col<eT> > P_fake_vector(fake_vector);
+  
+  
+  const char sig    = method[0];
+  const bool is_vec = (P.get_n_rows() == 1) || (P.get_n_cols() == 1);
+  
+  if(is_vec == true)
+    {
+    if( (sig == 'i') || (sig == 'I') || (sig == '+') )   // max norm
+      {
+      return arma_vec_norm_max(P_fake_vector);
+      }
+    else
+    if(sig == '-')   // min norm
+      {
+      const T val = arma_vec_norm_min(P_fake_vector);
+      
+      if( P.get_n_nonzero() < P.get_n_elem() )
+        {
+        return (std::min)(T(0), val);
+        }
+      else
+        {
+        return val;
+        }
+      }
+    else
+    if( (sig == 'f') || (sig == 'F') )
+      {
+      return arma_vec_norm_2(P_fake_vector);
+      }
+    else
+      {
+      arma_stop("norm(): unsupported vector norm type");
+      return T(0);
+      }
+    }
+  else
+    {
+    if( (sig == 'i') || (sig == 'I') || (sig == '+') )   // inf norm
+      {
+      return arma_mat_norm_inf(P);
+      }
+    else
+    if( (sig == 'f') || (sig == 'F') )
+      {
+      return arma_vec_norm_2(P_fake_vector);
+      }
+    else
+      {
+      arma_stop("norm(): unsupported matrix norm type");
+      return T(0);
+      }
+    }
+  }
+
+
+
 //! @}
