@@ -1,5 +1,5 @@
-// Copyright (C) 2008-2011 NICTA (www.nicta.com.au)
-// Copyright (C) 2008-2011 Conrad Sanderson
+// Copyright (C) 2008-2012 NICTA (www.nicta.com.au)
+// Copyright (C) 2008-2012 Conrad Sanderson
 // 
 // This file is part of the Armadillo C++ library.
 // It is provided without any warranty of fitness
@@ -103,6 +103,20 @@ op_sort::direct_sort(eT* X, const uword n_elem, const uword sort_type)
 
 
 
+template<typename eT>
+inline 
+void
+op_sort::direct_sort_ascending(eT* X, const uword n_elem)
+  {
+  arma_extra_debug_sigprint();
+  
+  arma_ascend_sort_helper<eT> comparator;
+    
+  std::sort(&X[0], &X[n_elem], comparator);
+  }
+
+
+
 template<typename eT>
 inline 
 void
@@ -158,8 +172,8 @@ op_sort::apply(Mat<typename T1::elem_type>& out, const Op<T1,op_sort>& in)
   
   typedef typename T1::elem_type eT;
   
-  const unwrap<T1>   tmp(in.m);
-  const Mat<eT>& X = tmp.M;
+  const unwrap_check<T1>   tmp(in.m, out);
+  const Mat<eT>&       X = tmp.M;
   
   const uword sort_type = in.aux_uword_a;
   const uword dim       = in.aux_uword_b;
